import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.List;

import bean.Author;
import bean.Book;
import bean.Publisher;
import utils.DbUtil;
import utils.system;

public class Application {
	private DbUtil db = DbUtil.createDBUtil();

	public static void main(String[] args) {
		new Application().run();
	}

	private void run() {
		System.out.println("欢迎使用图书管理系统 ");
		System.out.println("\t v1.0.0");
		System.out.println("--------系统菜单---------");
		System.out.println(" 1.检索全部图书");
		System.out.println(" 2.根据编号查询图书");
		System.out.println(" 3.添加图书");
		System.out.println(" 4.修改图书信息");
		System.out.println(" 5.删除图书");
		System.out.println(" 6.退出系统");
		System.out.println("--------------------------");
		try {
			int input = Integer.parseInt(system.input("请选择要进行的操作编号：")); // 获取输入
			if (input == 1) {
				// 全部
				showAllBook();
			} else if (input == 2) {
				// // 根据编号查询
				int id = Integer.parseInt(system.input("请输入要查询的图书编号：")); // 获取输入
				queryBook(id);
			} else if (input == 3) {
				// // 添加
				addBook();
			} else if (input == 4) {
				modifyBook();// 修改
			} else if (input == 5) {
				removeBook(); // 删除
			} else if (input == 6) {
				System.out.println("已经退出系统了");
				return; // 推出递归
			} else {
				System.err.println("选择错误");
			}
			run();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	
	
	private void removeBook() throws SQLException {
		// TODO Auto-generated method stub
		throw new SQLException("暂未实现此方法");		
	}

	private void modifyBook() throws SQLException {
		// TODO Auto-generated method stub
		throw new SQLException("暂未实现此方法");
	}



	private static SimpleDateFormat date = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	private void queryBook(int bookId) throws SQLException {
		Book b = db.get(Book.class, bookId);
		if (null == b) {
			system.println("查询的图书信息不存在");
			return;
		}
		system.println("编号:", b.getId());
		system.println("书名:", b.getName());
		system.println("作者:", b.getAuthor().getName());
		system.println("价格:", b.getId());
		system.println("出版社:", b.getPublisher().getName());
		system.println("入库时间:", date.format(b.getCreateTime()));
	}

	private void addBook() throws SQLException {
//		int id = Integer.parseInt(system.input("请输入图书编号:"));
		String name = system.input("请输入图书名称:");
		int author = getOrAddAuthor(system.input("请输入作者名称:"));
		float price = Float.parseFloat(system.input("请输入图书价格:"));
		int publisher = getOrAddPublisher(system.input("请输入图书出版社:"));
		
		int bookId = db.insert("insert into author (name,author,publisher,price,create_time,update_time) values (?,?,?,?,now(),now())", name,author,publisher,price);
		if(bookId > 0) {
			system.println("新增图书成功,编号为：",bookId);
		}else{
			system.error("添加图书失败");
		}
	}
	
	/**
	 * 获取作者的编号
	 * @param name
	 * @return
	 * @throws SQLException
	 */
	private int getOrAddAuthor(String name) throws SQLException {
		Author b = db.find(Author.class, "select * from author where name=?", name);
		if (b != null) {
			return b.getId();
		}
		return db.insert("insert into author (name,create_time,update_time) values (?,now(),now())", name);
	}
	
	/**
	 * 获取出版社的编号
	 * @param name
	 * @return
	 * @throws SQLException
	 */
	private int getOrAddPublisher(String name) throws SQLException {
		Publisher b = db.find(Publisher.class, "select * from publisher where name=?", name);
		if (b != null) {
			return b.getId();
		}
		return db.insert("insert into publisher (name,create_time,update_time) values (?,now(),now())", name);
	}

	private void showAllBook() throws SQLException {

		if (db.count("book") == 0) {
			system.error("还没有入库任何图书");
			return;
		}
		system.println("编号\t图书名称\t作者\t价格\t出版社");
		List<Book> list = db.queryList(Book.class);
		for (Book book : list) {
			system.println(book.getId(), '\t', book.getName(), '\t', book.getAuthor().getName(), '\t', book.getPrice(),
					'\t', book.getPublisher().getName());
		}
	}

}
