package utils;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

public class BeanProperty {
	public String name;
	public String field;
	public String type;
	public String pk = null;
	public boolean autoIncrement = false;
	public boolean fromSuper = false;

	public BeanProperty() {
	}

	public BeanProperty(String name, String field, String type) {
		this.name = name;
		this.field = field;
		this.type = type;
	}

	public BeanProperty(String name, String field, String type, boolean fromSuper) {
		this.name = name;
		this.field = field;
		this.type = type;
		this.fromSuper = fromSuper;
	}

	public static List<BeanProperty> getProperties(String className) throws ClassNotFoundException {
		List<BeanProperty> map = new ArrayList<BeanProperty>();
		Class<?> cls = Class.forName(className);

		Class<?> parent = cls.getSuperclass();
		parseFields(cls.getDeclaredFields(), map, false);

		if (parent.getName().equalsIgnoreCase("bean.BaseModel")) {
			parseFields(parent.getDeclaredFields(), map, true);
		}
		return map;
	}

	private static void parseFields(Field[] fs, List<BeanProperty> map, boolean fromSuper) {
		for (Field f : fs) {
			TableField tf = f.getAnnotation(TableField.class);
			if (tf != null) {
				BeanProperty bp = new BeanProperty(f.getName(), "".equals(tf.name()) ? f.getName() : tf.name(),
						f.getType().getName(), fromSuper);
				bp.pk = tf.relationKey();
				map.add(bp);
			}
		}
	}
}
