package utils;

import java.lang.reflect.Field;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DbUtil {
	static {
		try {
			Class.forName("com.mysql.jdbc.Driver");
		} catch (ClassNotFoundException e) {
			System.err.println("加载驱动失败:" + e.getMessage());
		}
	}

	private static Connection conn;
	private PreparedStatement ps;
	private ResultSet rs;
	// private static DbUtil dbUtil = null;

	public static DbUtil createDBUtil() {
		return createDBUtil("jdbc:mysql://127.0.0.1:3306/book", "root", "123456");
	}

	public static DbUtil createDBUtil(String url, String user, String pwd) {
		// if (dbUtil == null) {
		// dbUtil =;
		// }
		return new DbUtil(url, user, pwd);
	}

	private DbUtil(String url, String user, String pwd) {
		try {
			conn = DriverManager.getConnection(url, user, pwd);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public Map<String, String> getModel(String className) throws ClassNotFoundException {
		Map<String, String> map = new HashMap<String, String>();
		Class<?> cls = Class.forName(className);

		Class<?> parent = cls.getSuperclass();
		List<Field> fs = new ArrayList<Field>();
		fs.addAll(Arrays.asList(cls.getDeclaredFields()));
		if (parent.getName().equalsIgnoreCase("bean.BaseModel")) {
			fs.addAll(Arrays.asList(parent.getDeclaredFields()));
		}
		for (Field f : fs) {
			TableField tf = f.getAnnotation(TableField.class);
			if (tf != null) {
				map.put(f.getName(), "".equals(tf.name()) ? f.getName() : tf.name());
			}
		}
		return map;
	}

	public void dispose() {
		dispose(false);
	}

	public void dispose(boolean closeConnection) {
		if (null != rs) {
			try {
				rs.close();
				this.rs = null;
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		if (null != ps) {
			try {
				ps.close();
				this.ps = null;
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		if (closeConnection) {
			if (null != conn) {
				try {
					conn.close();
					conn = null;
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public <T> int save(String sql, Object... params) throws SQLException {
		prepareStatement(sql, params);
		int count = ps.executeUpdate();
		this.dispose();
		return count;
	}

	/**
	 * 插入数据并获取新增数据的自增ID
	 * 
	 * @param sql
	 * @param params
	 * @return
	 * @throws SQLException
	 */
	public <T> int insert(String sql, Object... params) throws SQLException {
		prepareStatement(sql, params);
		int count = ps.executeUpdate();
		if (count > 0) {
			// 获取自增id
			rs = ps.getGeneratedKeys();
			if (rs.next()) {
				return rs.getInt(1);
			}
		}
		this.dispose();
		return 0;
	}

	private void prepareStatement(String sql, Object... params) throws SQLException {
		ps = conn.prepareStatement(sql);
		if (params.length > 0) {
			for (int i = 0; i < params.length; i++) {
				ps.setObject(i + 1, params[i]);
			}
		}
	}

	private void executeQuery(String sql, Object... params) throws SQLException {
		prepareStatement(sql, params);
		rs = ps.executeQuery();
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public <T> T find(Class cls, String sql, Object... params) throws SQLException {
		executeQuery(sql, params);
		T t = null;
		if (rs.next()) {
			t = (T) convertToBean(cls);
		}
		this.dispose();
		return t;
	}

	@SuppressWarnings("unchecked")
	public <T> T get(Class<?> cls, Object id) throws SQLException {

		Table table = cls.getAnnotation(Table.class);
		if (table == null)
			return null;
		String tableName = table.name();
		if (tableName == null || "".equals(tableName)) {
			tableName = cls.getSimpleName();
		}
		String sql = "select * from " + tableName + " where " + table.pk() + "=?";
		executeQuery(sql, id);
		T t = null;
		if (rs.next()) {
			t = (T) convertToBean(cls);
		}
		this.dispose();
		return t;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public <T> List<T> queryList(Class cls, String sql, Object... params) throws SQLException {
		executeQuery(sql, params);
		List<T> ts = new ArrayList<T>();
		while (rs.next()) {
			ts.add((T) convertToBean(cls));
		}
		this.dispose();
		return ts;
	}

	@SuppressWarnings("unused")
	private boolean inArray(String[] arr, String str) {
		for (String s : arr) {
			if (str.equalsIgnoreCase(s)) {
				return true;
			}
		}
		return false;
	}

	// private static SimpleDateFormat date = new SimpleDateFormat("yyyy-MM-dd
	// HH:mm:ss");

	@SuppressWarnings("unused")
	private boolean isDBNull(ResultSet rs, String key) {
		return true;
	}

	private <T> Object getValueFromResult(BeanProperty p, Field f, T t) throws SQLException, ClassNotFoundException {
		// String[] basicType = { "int", "float", "short", "long", "java.lang.String" };
		String type = p.type.toLowerCase();
		if (type.equals("int")) { // 基本类型
			return rs.getInt(p.field);
		} else if (type.equals("float")) { // 基本类型
			return rs.getFloat(p.field);
		} else if (type.equals("double")) { // 基本类型
			return rs.getDouble(p.field);
		} else if (type.equals("short")) { // 基本类型
			return rs.getShort(p.field);
		} else if (type.equals("long")) { // 基本类型
			return rs.getLong(p.field);
		} else if (type.equals("java.lang.string")) { // 基本类型
			return rs.getString(p.field);
		} else if (type.equals("java.util.date")) { // 日期类型
			Object v = rs.getObject(p.field);
			if (null == v)
				return null;
			return new Date(rs.getDate(p.field).getTime());
		} else {
			String pk = p.pk;
			if (null == p.pk || "".equals(p.pk)) {
				pk = "id";
			}
			Class<?> cls = Class.forName(p.type);
			Table table = cls.getAnnotation(Table.class);
			if (table == null)
				return null;
			String tableName = table.name();
			if (tableName == null || "".equals(tableName)) {
				tableName = cls.getSimpleName();
			}
			return createDBUtil().find(cls, "select * from " + tableName + " where " + pk + "=?",
					rs.getObject(p.field));
		}
	}

	private <T> T convertToBean(Class<T> cls) throws SQLException {
		String name = cls.getName();
		T t = null;
		try {
			t = (T) cls.newInstance();
			List<BeanProperty> props = BeanProperty.getProperties(name);
			Class<?> parent = cls.getSuperclass();
			for (BeanProperty p : props) {
				Field f = p.fromSuper ? parent.getDeclaredField(p.name) : cls.getDeclaredField(p.name);
				f.setAccessible(true);
				f.set(t, getValueFromResult(p, f, t));
			}
		} catch (ClassNotFoundException e) {
			throw new SQLException("ClassNotFoundException:" + name);
		} catch (NoSuchFieldException e) {
			throw new SQLException("转换实体失败(没有找到对应字段 " + e.getMessage() + ")");
		} catch (SecurityException e) {
			throw new SQLException("SecurityException:" + e.getMessage());
		} catch (IllegalArgumentException e) {
			throw new SQLException("IllegalArgumentException:" + e.getMessage());
		} catch (IllegalAccessException e) {
			throw new SQLException("IllegalAccessException:" + e.getMessage());
		} catch (InstantiationException e) {
			throw new SQLException("InstantiationException:" + e.getMessage());
		}
		return t;
	}

	public Object getField(String sql, Object... params) throws SQLException {
		executeQuery(sql, params);
		if (rs.next()) {
			return rs.getObject(1);
		}
		return null;
	}

	public int count(String tableName) {
		int count = 0;
		try {
			count = (int) getField("select count(*) as total_count from " + tableName);
		} catch (SQLException e) {
			e.printStackTrace();
		} finally {
			this.dispose();
		}
		return count;
	}

	public <T> List<T> queryList(Class<T> cls) throws SQLException {
		String tableName = cls.getAnnotation(Table.class).name();
		if (tableName == null || "".equals(tableName)) {
			tableName = cls.getSimpleName();
		}
		return queryList(cls, "select * from " + tableName);
	}
}
